@@ -15,7 +15,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA.  */
 
 #include <config.h>
 
@@ -27,7 +27,7 @@
 #endif
 
 #include <stdio.h>
-#include "../memalloc.h"
+#include "memalloc.h"
 #include "../shell.h"
 #include "getopt.h"
 
