@@ -1,3 +1,7 @@
+/*
+ * print -- loadable ksh-93 style print builtin
+ */
+
 #include "bashtypes.h"
 
 #include <errno.h>
@@ -19,8 +23,6 @@ static int printargs ();
 
 static FILE *ofp;
 
-extern char *ansicstr ();
-
 extern char *this_command_name;
 
 static char *print_doc[] = {
@@ -150,7 +152,7 @@ printargs (list, ofp)
 
   for (sawc = 0, l = list; l; l = l->next)
     {
-      ostr = ansicstr (l->word->word, strlen (l->word->word), &sawc, (int *)0);
+      ostr = ansicstr (l->word->word, strlen (l->word->word), 0, &sawc, (int *)0);
       fprintf (ofp, "%s", ostr);
       free (ostr);
       if (sawc)
@@ -160,4 +162,3 @@ printargs (list, ofp)
     }
   return (1);
 }
-
