@@ -16,10 +16,10 @@
 
    You should have received a copy of the GNU General Public License along
    with Bash; see the file COPYING.  If not, write to the Free Software
-   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+   Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 
-#if !defined (__QUIT_H__)
-#define __QUIT_H__
+#if !defined (_QUIT_H_)
+#define _QUIT_H_
 
 /* Non-zero means SIGINT has already ocurred. */
 extern int interrupt_state;
@@ -30,4 +30,10 @@ extern void throw_to_top_level ();
    it is safe, put QUIT in the code, and the "interrupt" will take place. */
 #define QUIT if (interrupt_state) throw_to_top_level ()
 
-#endif /* __QUIT_H__ */
+#define SETINTERRUPT interrupt_state = 1
+#define CLRINTERRUPT interrupt_state = 0
+
+#define ADDINTERRUPT interrupt_state++
+#define DELINTERRUPT interrupt_state--
+
+#endif /* _QUIT_H_ */
